function Sim = SimSE(ParameterInputFile)

% SimSE		Numerical simulation of the solid-effect. Calculation of
% solid-effect enhancement profiles and time evolution of the polarization
% transfer for a two-spin system. The I spin is a proton!
%
% This routine was written dsitributed at the 2nd Winter School on
% Biomolecular solid-state NMR, January 24th-29th, 2010, Stowe, VT USA.
%
% Results = SimSE('InputFileName');
%
% Thorsten Maly, January 26th, 2010 (tmaly@mit.edu)

%% Define Constants

Const.betae = 1.399624580857917e+6;											% Bohr magneton [Hz/G] 
Const.giso = 2.0023193043718;												% Free electron g value
Const.gammae = Const.betae*Const.giso;										% Gamma electron [Hz/G]


%% Read paramaters from simulation input file

Sim.ParameterInputFile = ParameterInputFile;
eval(ParameterInputFile);													% Read input parameter file


%% Convert units
% In this section all input parameters are converted to standard units such
% as Hz, s, etc ...

Sim.A = SpinSys.A*1e6;														% Hypefine values in Hz
Sim.EulerA = SpinSys.EulerA/180*pi;											% Euler angles in radian
Sim.lwEPR = SpinSys.lwEPR*1e6;												% Linewidth in [Hz]
Sim.lwDNP = SpinSys.lwDNP*1e6;												% Linewidth in [Hz]
Sim.vMW = Par.vMW*1e9;														% Microwave frequency in [Hz]
Sim.w1 = Par.B1*1e6;														% B1 in [Hz]

Sim.phi = Par.phi/180*pi;													% SC Euler angles in radian
Sim.theta = Par.theta/180*pi;												% SC Euler angles in radian


%% Preallocate vectors, arrays, etc.
% In this section different vectors and arrays are pre-allocated to speed
% up calculations. Furthermore, to avoid strange behavior, observed at low
% resolution of the magnetic field vector the field resolution is checked
% against the EPR linewidth. If the difference is too large, the program
% terminates.

if Par.BnPts == 1
	Sim.B = Par.CF;															% Set field to center field for BnPts = 1
else
	Sim.B = linspace(Par.CF - Par.SW/2,Par.CF + Par.SW/2,Par.BnPts);		% Define field vector
	dB = sum(diff(Sim.B))/(length(Sim.B)-1)*10*2.804e6;						% Calculate field resolution in [Hz]
	if Sim.lwEPR*10 < dB													% Check linewidth
		disp('* ')
		disp('* ***** Attention ***** ')
		disp('* Linewidth parameter SpinSys.lwEPR much smaller than field resolution')
		disp(['* Current linewidth: ' num2str(Sim.lwEPR*1e-6) ' MHz'])
		disp(['* Current field resolution: ' num2str(dB) ' MHz'])
		disp('*')
		disp('* The program terminates here')
		disp('*')
		return
	end
end

if (strmatch(Opt.SC,'n') == 1)												
	[Sim.theta Sim.phi] = PowderOrientations(Par.SphereSlices,Par.Sym);		% Generate crystallite orientations for powder average
end

if length(Par.tp) == 1
    Sim.tp = Par.tp(1)*1e-6;												% Microwave evolution time in [s]
else
    Sim.tp = linspace(Par.tp(1),Par.tp(2),Par.tnPts)*1e-6;					% Calculate time evolutio vector [s]
end

Sim.EPRSpec = zeros(1,Par.BnPts);											% Preallocate vecotr for EPR spectrum
Sim.eDNP = zeros(Par.tnPts,Par.BnPts);										% Preallocate vecotr for DNP enhancement profile

SpinOp = SpinOperators(SpinSys.S, SpinSys.I);								% Generate spin operators for two-spin system.
Sim.Hilbert = prod(2*SpinSys.S+1)*prod(2*SpinSys.I+1);						% Dimension of the Hilber Space


%% Setup (static) Hamiltonians
% In this section Hamiltonians are calculated that remain constant during
% the calculation.

g = diag(SpinSys.g);														% Create g-tensor matrix
A = diag(Sim.A);															% Create A-tensor matrix

Sim.vI = 42580*Par.CF;														% Calculate nuclear Larmor frequency for 1H [Hz]
HI = Sim.vI.*SpinOp.Iz;														% Nuclear Zeeman interaction

R = RotMatPAS2ZAS([Sim.EulerA(1) Sim.EulerA(2) Sim.EulerA(3)]);				% Rotation matrix for hyperfine tensor
A = R*A/R;																	% Rotate hyperfine tensor into ZAS

H1 = Sim.w1*SpinOp.Sx;														% Microwave Hamiltonian


%% Calculate EPR Spectrum

if strcmpi(Opt.CalcEPR,'y') == 1
	SimSE_CalcEPR															% Calculate EPR spectrum
end


%% Calculate DNP field profile

if strcmpi(Opt.CalcDNP,'y') == 1
	SimSE_CalcDNP															% Calculate DNP profile	
end


%% Plot results
% This section calls a script to plot the simulation data. If no plot
% appears but the simulation didn't terminate with an error, no plot
% function is defined for this case. However, the simulation results are
% still available from the output variable or have to be loaded from the
% disk (if the safe option was set to 'y')

SimSE_PlotResults															% Create screen output

%% Save results
% If the save option is set to 'y' the results are saved on the disk after
% the simulation terminates. The filename has to be specified in the input
% script. The simulation results are saved with a .out extension. To load
% the data back into the workspace use Matlabs load function and specify
% the format with -mat
% Example: load FileName.out -mat

if strcmpi(Opt.Save,'y') == 1												% Save results

	eval(['save ' Opt.SaveFileName '.out Sim SpinSys Par Opt'])
	disp(['* Data saved as: ' Opt.SaveFileName '.out'])
	disp(' ')

end

%% Clean up

Sim.SpinOp = SpinOp;

end

%% Subfunctions
% All subfunctions required for the simulation are included in the main
% function SimSE.

function [theta phi] = PowderOrientations(n,sym)

% Based on kugel.m originally written by Jens Thoms Toerring (Frei
% University of Berlin, Germany)
% 
% [theta phi] = PowderOrientations(n)
% [theta phi] = PowderOrientations(n,sym)
% 
% Returns a set of theta and phi values for a given symmetry. If no
% symmetrie is specified values for Ci symmetrie are returned. Supported
% symmetries are: Ci, C2h, D2h, D4h

if nargin == 2
	switch sym
		
		case {'Ci'}
			SymPhi = 1;
		case {'C2h'}
			SymPhi = 2;
		case {'D2h'}
			SymPhi = 4;
		case {'D4h'}
			SymPhi = 8;
		otherwise
			disp('Symmetry not recognized, use Ci')
			SymPhi =1;
	end
else
	SymPhi = 1;
end

np = ceil(n*sin((0.5:n - 0.5)*pi/(2*n))/SymPhi);
nF = sum(np);

theta_j = 0;
count = 1;

for j = 1 : n,
	dtheta = acos(cos(theta_j) - np(j)/nF ) - theta_j;
	theta( count:count+4*np( j ) - 1) = theta_j + dtheta/2;
	dphi = pi/(2*SymPhi*np(j));
	phi(count:count+4*np(j) - 1) = 0.5*dphi:dphi:(4*np(j) - 0.5)*dphi;
	count = count + 4*np(j);
	theta_j = theta_j + dtheta;
end;

clear np cF theta_j count dtheta dphi;

end

function SpinOp = SpinOperators(S,I)

% Script to set up spinoperators for a two-spin system. Only spin operators
% that are needed for the numerical simulation are calculated.

mI = 2*I+1;
mS = 2*S+1;

EI = eye(mI);
ES = eye(mS);

SpinOp.E = kron(ES,EI);

SpinOp.Sx = kron(Jx(S),EI);
SpinOp.Sy = kron(Jy(S),EI);
SpinOp.Sz = kron(Jz(S),EI);

SpinOp.Ix = kron(ES,Jx(I));
SpinOp.Iy = kron(ES,Jy(I));
SpinOp.Iz = kron(ES,Jz(I));

SpinOp.SzIx = kron(Jz(S),Jx(I));
SpinOp.SzIy = kron(Jz(S),Jy(I));
SpinOp.SzIz = kron(Jz(S),Jz(I));
end

function Out = Jz(j)
% General spin operator Jz
% Input: Jx(j) with j = 1/2, 1, 3/2, ....
% With no input argument the j = 1/2 operator is calculated
% last change: tm, 16.07.03

if nargin == 0
	j = 1/2;	
end

Mult = round(2*j+1);
Out = zeros(Mult,Mult);

for k =-j:j	
	Out(k+j+1,k+j+1)= - k;
end
end

function Out = Jy(j)
% General spin operator Jy
% Input: Jx(j) with j = 1/2, 1, 3/2, ....
% With no input argument the j = 1/2 operator is calculated
% last change: tm, 16.07.03

if nargin == 0
	j = 1/2;
end

Out = -0.5*1i*(Jp(j)-Jm(j));
end

function Out = Jx(j)
% General spin operator Jx
% Input: Jx(j) with j = 1/2, 1, 3/2, ....
% With no input argument the j = 1/2 operator is calculated
% last change: tm, 16.07.03

if nargin == 0
	j = 1/2;	
end

Out = 0.5*(Jp(j)+Jm(j));
end

function Out = Jp(j)
% General spin operator J_plus
% Input: Jp(j) with j = 1/2, 1, 3/2, ....
% With no input argument the j = 1/2 operator is calculated
% last change: tm, 16.07.03

if nargin == 0	
	j = 1/2;
end

Mult = round(2*j+1);
Out = zeros(Mult,Mult);

for k =-j:j-1
	Out(k+j+1,k+j+2)= sqrt(j*(j+1) - k*(k+1));	
end
end

function Out = Jm(j)
% General spin operator J_minus
% Input: Jm(j) with j = 1/2, 1, 3/2, ....
% With no input argument the j = 1/2 operator is calculated
% last change: tm, 16.07.03

if nargin == 0
	j = 1/2;
end

Mult = round(2*j+1);
Out = zeros(Mult,Mult);

for k =-j+1:j
	Out(k+j+1,k+j)= sqrt(j*(j+1) - k*(k-1));
end
end

function R = RotMatPAS2ZAS(Euler)

% Calculate rotation matrix to rotate tensors to central axis system (ZAS)
% (zyz convention from Rose)
%
% RotPAS2ZAS([alpha beta gamma], Matrix)
%
% Based on definition of Rose
%
% alpha   --> phi
% beta    --> theta
% gamma   --> psi
%
% Input: Euler angles (in rad)
%
% tm, 25.04.03

R(1,1) = cos(Euler(3))*cos(Euler(2))*cos(Euler(1)) - sin(Euler(3))*sin(Euler(1));
R(1,2) = cos(Euler(3))*cos(Euler(2))*sin(Euler(1)) + sin(Euler(3))*cos(Euler(1));
R(1,3) = -cos(Euler(3))*sin(Euler(2));
R(2,1) = -sin(Euler(3))*cos(Euler(2))*cos(Euler(1)) - cos(Euler(3))*sin(Euler(1));
R(2,2) = -sin(Euler(3))*cos(Euler(2))*sin(Euler(1)) + cos(Euler(3))*cos(Euler(1));
R(2,3) = sin(Euler(3))*sin(Euler(2));
R(3,1) = sin(Euler(2))*cos(Euler(1));
R(3,2) = sin(Euler(2))*sin(Euler(1));
R(3,3) = cos(Euler(2));

end

function out = ConvoluteLw(x,y,lw)

% Convolution of an input vector y with a linewidth (Lorentzian). The
% convolution is performed by an inverse fourier transformation.
%
% Out = ConvoluteLw(x,y,lw)
%
% With y the input vector and lw the linewidth parameter. A gaussian
% lineshape function is used for convolution. Note that the linewidht
% parameter has the same unit as the x vector.
%
% tm, Aug. 2008

if size(x,1) ~= 1
	x = x';
end

if size(y,1) ~= 1
	y = y';
end

fftlw = fft(2/(pi*sqrt(3))*(1/lw)./(1+(4/3)*(((x-mean(x))/lw).^2)));		% Calculate fourrier tranformed of lineshape function
out = real(fftshift(ifft(fft(y).*fftlw)));									% Multiplication of spektrum and lineshape function and invers fourrier transformation

out = max(y)*out/max(out);
end